@@ -64,6 +64,25 @@ func printProblemsHeader() {
 	fmt.Fprintln(os.Stderr, "EDITOR       :", os.Getenv("EDITOR"))
 	fmt.Fprintln(os.Stderr, "TERM_PROGRAM :", os.Getenv("TERM_PROGRAM"))
 	fmt.Fprintln(os.Stderr)
+
+	lessenv_section := ""
+	lessTermcapVars := []string{
+		"LESS_TERMCAP_md",
+		"LESS_TERMCAP_us",
+		"LESS_TERMCAP_so",
+	}
+	for _, varName := range lessTermcapVars {
+		value := os.Getenv(varName)
+		if value == "" {
+			continue
+		}
+
+		lessenv_section += varName + " : " + strings.ReplaceAll(value, "\x1b", "ESC") + "\n"
+	}
+	if lessenv_section != "" {
+		fmt.Fprintln(os.Stderr, lessenv_section)
+	}
+
 	fmt.Fprintln(os.Stderr, "GOOS    :", runtime.GOOS)
 	fmt.Fprintln(os.Stderr, "GOARCH  :", runtime.GOARCH)
 	fmt.Fprintln(os.Stderr, "Compiler:", runtime.Compiler)
@@ -149,7 +168,7 @@ func parseUnprintableStyle(styleOption string) (textstyles.UnprintableStyleT, er
 	return 0, fmt.Errorf("Good ones are highlight or whitespace")
 }
 
-func parseScrollHint(scrollHint string) (twin.StyledRune, error) {
+func parseScrollHint(scrollHint string) (textstyles.CellWithMetadata, error) {
 	scrollHint = strings.ReplaceAll(scrollHint, "ESC", "\x1b")
 	hintAsLine := reader.NewLine(scrollHint)
 	parsedTokens := hintAsLine.HighlightedTokens(twin.StyleDefault, twin.StyleDefault, nil, nil, nil).StyledRunes
@@ -157,7 +176,7 @@ func parseScrollHint(scrollHint string) (twin.StyledRune, error) {
 		return parsedTokens[0], nil
 	}
 
-	return twin.StyledRune{}, fmt.Errorf("Expected exactly one (optionally highlighted) character. For example: 'ESC[2m…'")
+	return textstyles.CellWithMetadata{}, fmt.Errorf("Expected exactly one (optionally highlighted) character. For example: 'ESC[2m…'")
 }
 
 func parseShiftAmount(shiftAmount string) (uint, error) {
@@ -330,6 +349,14 @@ func noLineNumbersDefault() bool {
 		return true
 	}
 
+	if os.Getenv("GIT_EXEC_PATH") != "" {
+		// Set by "git".
+
+		// Neither logs nor diffs are helped by line numbers, turn them off by
+		// default.
+		return true
+	}
+
 	// Default to not skipping line numbers
 	return false
 }
@@ -396,10 +423,10 @@ func pagerFromArgs(
 	unprintableStyle := flagSetFunc(flagSet, "render-unprintable", textstyles.UnprintableStyleHighlight,
 		"How unprintable characters are rendered: highlight or whitespace", parseUnprintableStyle)
 	scrollLeftHint := flagSetFunc(flagSet, "scroll-left-hint",
-		twin.NewStyledRune('<', twin.StyleDefault.WithAttr(twin.AttrReverse)),
+		textstyles.CellWithMetadata{Rune: '<', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
 		"Shown when view can scroll left. One character with optional ANSI highlighting.", parseScrollHint)
 	scrollRightHint := flagSetFunc(flagSet, "scroll-right-hint",
-		twin.NewStyledRune('>', twin.StyleDefault.WithAttr(twin.AttrReverse)),
+		textstyles.CellWithMetadata{Rune: '>', Style: twin.StyleDefault.WithAttr(twin.AttrReverse)},
 		"Shown when view can scroll right. One character with optional ANSI highlighting.", parseScrollHint)
 	shift := flagSetFunc(flagSet, "shift", 16, "Horizontal scroll `amount` >=1, defaults to 16", parseShiftAmount)
 	tabSize := flagSetFunc(flagSet, "tab-size", 8, "Number of spaces per tab stop, defaults to 8", parseTabAmount)
@@ -528,6 +555,14 @@ func pagerFromArgs(
 	shouldFormat := *reFormat
 	readerOptions := reader.ReaderOptions{Lexer: *lexer, ShouldFormat: shouldFormat}
 
+	stdinName := ""
+	if os.Getenv("PAGER_LABEL") != "" {
+		stdinName = os.Getenv("PAGER_LABEL")
+	} else if os.Getenv("MAN_PN") != "" {
+		// MAN_PN is set by GNU man. Example value: "printf(1)"
+		stdinName = os.Getenv("MAN_PN")
+	}
+
 	// Display the input file(s) contents
 	stdinDone := false
 	for _, inputFilename := range flagSetArgs {
@@ -540,7 +575,7 @@ func pagerFromArgs(
 				continue
 			}
 
-			readerImpl, err = reader.NewFromStream("<stdin>", os.Stdin, formatter, readerOptions)
+			readerImpl, err = reader.NewFromStream(stdinName, os.Stdin, formatter, readerOptions)
 			if err != nil {
 				return nil, nil, chroma.Style{}, nil, logsRequested, err
 			}
@@ -695,10 +730,7 @@ func startPaging(pager *internal.Pager, screen twin.Screen, chromaStyle *chroma.
 		}
 
 		if !pager.DeInit {
-			err := pager.ReprintAfterExit()
-			if err != nil {
-				log.Error("Failed reprinting pager view after exit: ", err)
-			}
+			pager.ReprintAfterExit()
 		}
 
 		if pager.AfterExit != nil {
