@@ -11,19 +11,59 @@ type CellWithMetadata struct {
 	Rune  rune
 	Style twin.Style
 
+	cachedWidth *int
+
 	StartsSearchHit bool // True if this cell is the first cell of a search hit
 }
 
+// Required for some tests to pass
+func (r CellWithMetadata) Equal(b CellWithMetadata) bool {
+	if r.Rune != b.Rune {
+		return false
+	}
+
+	if !r.Style.Equal(b.Style) {
+		return false
+	}
+
+	if r.StartsSearchHit != b.StartsSearchHit {
+		return false
+	}
+
+	return true
+}
+
 func (r CellWithMetadata) ToStyledRune() twin.StyledRune {
 	return twin.NewStyledRune(r.Rune, r.Style)
 }
 
-func (r CellWithMetadata) Width() int {
-	return r.ToStyledRune().Width()
+func (r *CellWithMetadata) Width() int {
+	if r.cachedWidth != nil {
+		return *r.cachedWidth
+	}
+
+	// Cache it
+	w := r.ToStyledRune().Width()
+	r.cachedWidth = &w
+	return w
 }
 
 type CellWithMetadataSlice []CellWithMetadata
 
+func (runes CellWithMetadataSlice) Equal(other CellWithMetadataSlice) bool {
+	if len(runes) != len(other) {
+		return false
+	}
+
+	for i := range runes {
+		if !runes[i].Equal(other[i]) {
+			return false
+		}
+	}
+
+	return true
+}
+
 // Returns a copy of the slice with leading whitespace removed
 func (runes CellWithMetadataSlice) WithoutSpaceLeft() CellWithMetadataSlice {
 	for i := range runes {
